/**
 * Proyecto: 9x03-Bicola - Referencias y Ficheros
 * Fichero:  Bicola.java
 * Fecha:    24/05/2012
 *
 * Licencia (CC) BY-NC-SA: giltesa.com
 * 
 * http://josedeveloper.com/2012/02/24/sabias-que-el-serialversionuid-en-las-clases-java-nos-previene-de-errores-en-la-deserializacion/
 */


// import java.io.File;						// Para trabajar con ficheros.
import java.io.FileNotFoundException;
import java.io.ObjectInputStream;	// Para leer objetos desde ficheros.
import java.io.FileInputStream;		// Revisar: Que son cada uno??
import java.io.FileOutputStream;
import java.io.ObjectOutputStream;	// Para grabar objetos a ficheros.
import java.io.Serializable;			// Para permitir que un objeto pueda ser leido/grabado en un fichero.


public class Bicola<G> implements Serializable
{

	// PROPIEDADES:
	private int	              numDeNodos;
	private Nodo<G>	        primerNodo;
	private Nodo<G>	        ultimoNodo;
	private static final long	serialVersionUID	= -3054441480627810268L; // "Identificador de la clase"


	// METODOS:
	public int getNumDeNodos()
	{
		return numDeNodos;
	}


	// Aade un nuevo Nodo a la izquierda modificando las referencias de las propiedades.
	public void addDatoIzquierda( G dato )
	{
		Nodo<G> temp = new Nodo<G>(dato);

		if( primerNodo == null )
		{
			primerNodo = temp;
			ultimoNodo = temp;
		}
		else
		{
			temp.setNodoSiguiente(primerNodo);
			primerNodo.setNodoAnterior(temp);
			primerNodo = temp;
		}
		numDeNodos++;
	}


	// Aade un nuevo Nodo a la derecha modificando las referencias de las propiedades.
	public void addDatoDerecha( G dato )
	{
		Nodo<G> temp = new Nodo<G>(dato);

		if( ultimoNodo == null )
		{
			primerNodo = temp;
			ultimoNodo = temp;
		}
		else
		{
			temp.setNodoAnterior(ultimoNodo);
			ultimoNodo.setNodoSiguiente(temp);
			ultimoNodo = temp;
		}
		numDeNodos++;
	}


	// Extrae el primer dato y cambia las referenciaas de las propiedades.
	public G getDatoIzquierda()
	{
		G temp = primerNodo.getDatoNodo();

		if( primerNodo == ultimoNodo ) // Solo hay un nodo y se acaba de extraer quedando ahora 0.
		{
			primerNodo = null;
			ultimoNodo = null;
		}
		else
		{
			primerNodo = primerNodo.getNodoSiguiente();
			primerNodo.setNodoAnterior(null);
		}

		numDeNodos--;
		return temp;
	}


	// Extrae el ultimo dato y cambia las referenciaas de las propiedades.
	public G getDatoDerecha()
	{
		G temp = ultimoNodo.getDatoNodo();

		if( primerNodo == ultimoNodo )
		{
			primerNodo = null;
			ultimoNodo = null;
		}
		else
		{
			ultimoNodo = ultimoNodo.getNodoAnterior();
			ultimoNodo.setNodoSiguiente(null);
		}

		numDeNodos--;
		return temp;
	}


	// Concatena todos los elementos de la Bicola por referencias y lo retorna.
	public String toString()
	{
		String cadena = "";
		Nodo<G> posicion = primerNodo;

		while(posicion != null)
		{
			cadena += posicion.getDatoNodo() + ", ";
			posicion = posicion.getNodoSiguiente();
		}
		return cadena;
	}


	public Bicola<G> readBicolaFichero()
	{
		FileInputStream fichEntrada = null; // Revisar: Esto sobra?
		ObjectInputStream objEntrada = null;
		Bicola<G> temp = null;


		try
		{
			fichEntrada = new FileInputStream("backup.dat");
			objEntrada = new ObjectInputStream(fichEntrada);
			temp = (Bicola<G>) objEntrada.readObject();
		}
		catch( FileNotFoundException e )
		{
			System.out.println("El sistema no puede encontrar el archivo especificado");
		}
		catch( ClassNotFoundException e )
		{
			System.out.println("El contenido leido en \"backup.dat\" no era el esperado.");
		}
		catch( Exception e )
		{
			System.out.println(e);
		}
		finally
		{
			// Como ultimo paso se cierran los ficheros.
			try
			{
				if( objEntrada != null )
					objEntrada.close();
				if( fichEntrada != null )
					fichEntrada.close();
			}
			catch( Exception e )
			{
				System.out.println(e);
			}

		}

		return temp;
	}


	public void writeBicolaFichero( Bicola<G> bicola )
	{
		FileOutputStream fichSalida = null;
		ObjectOutputStream objSalida = null;

		try
		{
			fichSalida = new FileOutputStream("backup.dat");
			objSalida = new ObjectOutputStream(fichSalida);
			objSalida.writeObject(bicola);
		}
		catch( FileNotFoundException e )
		{
			System.out.println("El sistema no puede encontrar el archivo especificado");
		}
		catch( Exception e )
		{
			System.out.println(e);
		}
		finally
		{
			// Como ultimo paso se cierran los ficheros.
			try
			{
				if( objSalida != null )
					objSalida.close();
				if( fichSalida != null )
					fichSalida.close();
			}
			catch( Exception e )
			{
				System.out.println("No se pudo cerrar el fichero.");
			}

		}
	}

}
